import https from "node:https";
import path from "path";
import fs from "fs";
import grabHostNames from "../utils/grab-host-names";
import apiGet from "../functions/api/query/get";
import serializeQuery from "../utils/serialize-query";
import {
    ApiGetQueryObject,
    DSQL_DatabaseSchemaType,
    GetReqQueryObject,
    GetReturn,
} from "../types";
import apiGetGrabQueryAndValues from "../utils/grab-query-and-values";
import debugLog from "../utils/logging/debug-log";

type Param<T extends { [k: string]: any } = { [k: string]: any }> = {
    key?: string;
    db?: string;
    query: string | ApiGetQueryObject<T>;
    queryValues?: string[];
    tableName?: string;
    user_id?: string | number;
    debug?: boolean;
    forceLocal?: boolean;
};

export type ApiGetParams = Param;

/**
 * # Make a get request to Datasquirel API
 */
export default async function get<
    T extends { [k: string]: any } = { [k: string]: any },
    R extends any = any
>({
    key,
    db,
    query,
    queryValues,
    tableName,
    user_id,
    debug,
    forceLocal,
}: Param<T>): Promise<GetReturn<R>> {
    const grabedHostNames = grabHostNames();
    const { host, port, scheme } = grabedHostNames;

    function debugFn(log: any, label?: string) {
        debugLog({ log, addTime: true, title: "apiGet", label });
    }

    /**
     * Check for local DB settings
     *
     * @description Look for local db settings in `.env` file and by pass the http request if available
     */
    const { DSQL_DB_NAME } = process.env;

    if (DSQL_DB_NAME?.match(/./) && global.DSQL_USE_LOCAL) {
        let dbSchema: DSQL_DatabaseSchemaType | undefined;

        try {
            const localDbSchemaPath = path.resolve(
                process.cwd(),
                "dsql.schema.json"
            );
            dbSchema = JSON.parse(fs.readFileSync(localDbSchemaPath, "utf8"));
        } catch (error) {}

        if (debug) {
            debugFn("Running Locally ...");
        }

        return await apiGet({
            dbFullName: DSQL_DB_NAME,
            query,
            queryValues,
            tableName,
            dbSchema,
            debug,
            forceLocal,
        });
    }

    /**
     * Make https request
     *
     * @description make a request to datasquirel.com
     */
    const httpResponse = await new Promise((resolve, reject) => {
        const queryAndValues = apiGetGrabQueryAndValues({
            query,
            values: queryValues,
        });

        const queryObject: GetReqQueryObject = {
            db: process.env.DSQL_API_DB_NAME || String(db),
            query: queryAndValues.query,
            queryValues: queryAndValues.valuesString,
            tableName,
            debug,
        };

        if (debug) {
            debugFn(queryObject, "queryObject");
        }

        const queryString = serializeQuery({ ...queryObject });

        if (debug) {
            debugFn(queryString, "queryString");
        }

        let path = `/api/query/${
            user_id || grabedHostNames.user_id
        }/get${queryString}`;

        if (debug) {
            debugFn(path, "path");
        }

        const requestObject: https.RequestOptions = {
            method: "GET",
            headers: {
                "Content-Type": "application/json",
                Authorization:
                    key ||
                    process.env.DSQL_READ_ONLY_API_KEY ||
                    process.env.DSQL_FULL_ACCESS_API_KEY ||
                    process.env.DSQL_API_KEY,
            },
            port,
            hostname: host,
            path,
        };

        scheme
            .request(
                requestObject,

                /**
                 * Callback Function
                 *
                 * @description https request callback
                 */
                (response) => {
                    var str = "";

                    response.on("data", function (chunk) {
                        str += chunk;
                    });

                    response.on("end", function () {
                        try {
                            resolve(JSON.parse(str) as GetReturn);
                        } catch (/** @type {any} */ error: any) {
                            reject({
                                error: error.message,
                                result: str,
                            });
                        }
                    });

                    response.on("error", (err) => {
                        console.log("DSQL get Error,", err.message);
                        resolve(null);
                    });
                }
            )
            .end();
    });

    return httpResponse as GetReturn;
}
