import fs from "fs";
import grabDirNames from "../../utils/backend/names/grab-dir-names";
import {
    DSQL_DatabaseSchemaType,
    DSQL_FieldSchemaType,
    DSQL_TableSchemaType,
} from "../../types";
import _ from "lodash";
import EJSON from "../../utils/ejson";
import generateTypeDefinition from "./generate-type-definitions";
import { AppNames } from "../../dict/app-names";

type Params = {
    dbSchema?: DSQL_DatabaseSchemaType;
};

export default function dbSchemaToType(params?: Params): string[] | undefined {
    let datasquirelSchema;
    const { mainShemaJSONFilePath, defaultTableFieldsJSONFilePath } =
        grabDirNames();

    if (params?.dbSchema) {
        datasquirelSchema = params.dbSchema;
    } else {
        const mainSchema = EJSON.parse(
            fs.readFileSync(mainShemaJSONFilePath, "utf-8")
        ) as DSQL_DatabaseSchemaType[];

        datasquirelSchema = mainSchema.find(
            (sch) => sch.dbFullName == "datasquirel"
        );
    }

    if (!datasquirelSchema) return;

    let tableNames = `export const DsqlTables = [\n${datasquirelSchema.tables
        .map((tbl) => `    "${tbl.tableName}",`)
        .join("\n")}\n] as const`;

    const defaultFields = EJSON.parse(
        fs.readFileSync(defaultTableFieldsJSONFilePath, "utf-8")
    ) as DSQL_FieldSchemaType[];

    const dbTablesSchemas = datasquirelSchema.tables.map((tblSchm) => {
        let newDefaultFields = _.cloneDeep(defaultFields);
        return {
            ...tblSchm,
            fields: tblSchm.fields.find((fld) => fld.fieldName == "id")
                ? tblSchm.fields
                : [
                      newDefaultFields.shift(),
                      newDefaultFields.shift(),
                      ...tblSchm.fields,
                      ...newDefaultFields,
                  ],
        } as DSQL_TableSchemaType;
    });

    const defDbName = (
        datasquirelSchema.dbName ||
        datasquirelSchema.dbFullName?.replace(
            new RegExp(`${AppNames["DsqlDbPrefix"]}\\d+_`),
            ""
        )
    )
        ?.toUpperCase()
        .replace(/ /g, "_");

    const schemas = dbTablesSchemas
        .map((table) =>
            generateTypeDefinition({
                paradigm: "TypeScript",
                table,
                typeDefName: `DSQL_${defDbName}_${table.tableName.toUpperCase()}`,
                allValuesOptional: true,
                addExport: true,
            })
        )
        .filter((schm) => typeof schm == "string");

    return [tableNames, ...schemas];
}
